import * as path from 'path';
import { tscTask, argv } from 'just-scripts';

const libPath = path.resolve(process.cwd(), 'lib');
const srcPath = path.resolve(process.cwd(), 'src');

const useTsBuildInfo = false

function getExtraTscParams(args) {
    return {
        pretty: true,
        target: 'es5',
        // sourceMap must be true for inlineSources and sourceRoot to work
        ...(args.production && { inlineSources: true, sourceRoot: path.relative(libPath, srcPath), sourceMap: true }),
    };
}

export const ts = {
    commonjs: () => {
        const extraOptions = getExtraTscParams(argv());
        return tscTask({
            ...extraOptions,
            outDir: 'lib-commonjs',
            module: 'commonjs',
            // @ts-ignore
            ...(useTsBuildInfo && { tsBuildInfoFile: '.commonjs.tsbuildinfo' }),
        });
    },
    esm: () => {
        const extraOptions = getExtraTscParams(argv());
        // Use default tsbuildinfo for this variant
        return tscTask({ ...extraOptions, outDir: 'lib', module: 'esnext' });
    },
    amd: () => {
        const extraOptions = getExtraTscParams(argv());
        return tscTask({
            ...extraOptions,
            outDir: 'lib-amd',
            module: 'amd',
            // @ts-ignore
            ...(useTsBuildInfo && { tsBuildInfoFile: '.amd.tsbuildinfo' }),
        });
    },
    commonjsOnly: () => {
        const extraOptions = getExtraTscParams(argv());
        // Use default tsbuildinfo for this variant (since it's the only variant)
        return tscTask({ ...extraOptions, outDir: 'lib', module: 'commonjs' });
    },
};